--[[
	BarOptions
	 	Lets you configure extra options for the Main Bar and Action bars.

	By: Mugendai
	Contact: mugekun@gmail.com

	Gives several options for configuration of the varying bars:
	Remove art from the bars
	Add background art to the multi bars
	Provide coloring of actions that are not in range of their target.
	Use a different set of ID ranges for the bars, like secondbar, and sidebar did.
	Group the bottom left bar and action bar, for turning pages, like secondbar did.
	Make the bottom left bar use different ID's based on stance.
	Show/Hide empty bar buttons.
	Show the hotkeys on the multi bars.
	Show/Hide all hotkeys.
	Shorten the names of the modifier keys for bindings.
	Change the number of buttons on each of the four multi bars.

	$Id: BarOptions.lua 4717 2007-02-16 18:22:15Z Gryphon $
	$Rev: 4717 $
	$LastChangedBy: Gryphon $
	$Date: 2007-02-16 12:22:15 -0600 (Fri, 16 Feb 2007) $
]]--

--------------------------------------------------
--
-- Constants
--
--------------------------------------------------
ALT_BOTTOMLEFT_ACTIONBAR_PAGE = 2;
ALT_BOTTOMRIGHT_ACTIONBAR_PAGE = 6;
ALT_SIDEBAR_PAGE = 3;

--------------------------------------------------
--
-- BarOptions Declaration
--
--------------------------------------------------
BarOptions = {
	--------------------------------------------------
	--
	-- Constants
	--
	--------------------------------------------------
	VERSION = 1.40;
	BARLIST = {
		"Action";
		"BonusAction";
		"MultiBarBottomLeft";
		"MultiBarBottomRight";
		"MultiBarRight";
		"MultiBarLeft";
	};
};
--Main Configuration variable
BarOptions_Config = {};
BarOptions.SetDefaults = function ()
	BarOptions_Config.AlternateIDs = 0; --Whether to use the alternate button ID ranges or not
	BarOptions_Config.TurnPages = 0; --Whether bottom left multi bar changes pages when the page changes
	BarOptions_Config.GroupPages = 0; --Whether to use grouped pages for mainbar and bottom left multi bar
	BarOptions_Config.StanceBar = 0; --Whether the bottom left multibar should switch to an different page when stance changes
	BarOptions_Config.DruidStealth = 1; --Whether to use one extra bar for both feral forms, and the other as a cat stealth bar
	BarOptions_Config.HideEmpty = 1; --Whether to hide empty buttons or not
	BarOptions_Config.MainBarCenter = 0; --Whether to move the main bar to the bottom center or not
	BarOptions_Config.MainBarLeft = 0; --Whether to move the main bar to the bottom left or not
	BarOptions_Config.MainBarRight = 0; --Whether to move the main bar to the bottom right or not
	BarOptions_Config.HideKeys = 0; --Whether to hide all hotkeys or not
	BarOptions_Config.MultiKeys = 0; --Whether to show hotkeys on the multi bars or not
	BarOptions_Config.ShortKeys = 0; --Whether to shorten the names of the key modifiers or not
	BarOptions_Config.HideKeyMod = 0; --Whether to hide the hotkey modifiers or not
	BarOptions_Config.MainArt = 1; --Whether to show the main bar artwork or not
	BarOptions_Config.BLArt = 0; --Whether to show the bottom left bar artwork or not
	BarOptions_Config.BRArt = 0; --Whether to show the bottom right bar artwork or not
	BarOptions_Config.RArt = 0; --Whether to show the right bar artwork or not
	BarOptions_Config.LArt = 0; --Whether to show the left bar artwork or not
	BarOptions_Config.BLCount = 12; --HoBarOptions_Config.w many buttons on the bottom left multi bar
	BarOptions_Config.BRCount = 12; --How many buttons on the bottom right multi bar
	BarOptions_Config.RCount = 12; --How many buttons on the right multi bar
	BarOptions_Config.LCount = 12; --How many buttons on the left multi bar
	BarOptions_Config.RangeColorOn = 0;	--Should we color action buttons when they arent in range
	BarOptions_Config.RangeColor = {	r = 1;			--How much red
																		g = 0.5;		--How much green
																		b = 0.5; };	--How much blue
	BarOptions_Config.CustomStances = 0; --If anabled then the customized stance pages will be used.
	BarOptions_Config.Stance0 = 6; --No stance page
	BarOptions_Config.Stance1 = 3; --Stance numer 1 page
	BarOptions_Config.Stance2 = 2; --Stance numer 2 page
	BarOptions_Config.Stance3 = 10; --Stance numer 3 page
end;
BarOptions.SetDefaults();
--Store this config for safe load
MCom.safeLoad("BarOptions_Config");

--------------------------------------------------
--
-- Private Functions
--
--------------------------------------------------
--[[ Sets things back to their defaults ]]--
BarOptions.DefaultSetup = function ()
	--Set the default settings
	BarOptions.SetDefaults();
	--Update the UI with the defaults
	MCom.updateUI("/baroptions");
	--Update the varying UI elements
	BarOptions.UpdateHotkeys();
	BarOptions.ShowMultiHotkeys();
	BarOptions.UpdateButtons();
	--Move the right and left multi bars to their normal positions
	MultiBarLeft:ClearAllPoints();
	MultiBarLeft:SetPoint("TOPRIGHT", "MultiBarRight", "TOPLEFT", -5, 0);
	MultiBarLeft:SetUserPlaced(false);
	MultiBarRight:ClearAllPoints();
	MultiBarRight:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMRIGHT", -7, 98);
	MultiBarRight:SetUserPlaced(false);
	MultiActionBar_Update();
	--Move the Party Members frame to its default position
	BarOptions.OffsetParty();
	--Set MobileFrames Offsets if availible.
	if(MobileFrames_SetOffsets) then
		MobileFrames_SetOffsets("REGULAR 0");
		MobileFrames_SetOffsets("CONTAINERS 90");
	end
	--Print output to let the player know the command succeeded, if there is no UI
	MCom.printStatus(BAROPTIONS_CHAT_DEFAULTSETUP);
end;

--[[ Sets things up to behave like Sidebars ]]--
BarOptions.SideBarSetup = function ()
	--Set to the sidebar settings
	BarOptions_Config.AlternateIDs = 1;
	BarOptions_Config.TurnPages = 1;
	BarOptions_Config.MultiKeys = 1;
	BarOptions_Config.LCount = 6;
	BarOptions_Config.RCount = 6;
	BarOptions_Config.RangeColorOn = 1;
	--Update the UI with the defaults
	MCom.updateUI("/baroptions");
	--Update the varying UI elements
	BarOptions.UpdateHotkeys();
	BarOptions.ShowMultiHotkeys();
	BarOptions.UpdateButtons();
	--Move the right and left multi bars to their sidebar positions
	MultiBarLeft:ClearAllPoints();
	MultiBarLeft:SetPoint("TOPLEFT", "UIParent", "LEFT", -5, (98 - 80) + 56);
	MultiBarLeft:SetUserPlaced(true);
	MultiBarRight:ClearAllPoints();
	MultiBarRight:SetPoint("TOPRIGHT", "UIParent", "RIGHT", 0, 98 + 56);
	MultiBarRight:SetUserPlaced(true);

	--Move the Party Members frame out of the way of the left bar
	BarOptions.OffsetParty(true);

	--Set MobileFrames Offsets if availible.
	if(MobileFrames_SetOffsets) then
		MobileFrames_SetOffsets("REGULAR 40");
		MobileFrames_SetOffsets("CONTAINERS 30");
	end

	--Enable the sidebars
	SHOW_MULTI_ACTIONBAR_3 = 1;
	SHOW_MULTI_ACTIONBAR_4 = 1;
	--Save sidebar state
	SetActionBarToggles(SHOW_MULTI_ACTIONBAR_1, SHOW_MULTI_ACTIONBAR_2, SHOW_MULTI_ACTIONBAR_3, SHOW_MULTI_ACTIONBAR_4);

	MultiActionBar_Update();
	UIParent_ManageRightSideFrames();
	FCF_UpdateCombatLogPosition();
	UIOptionsFrame_UpdateDependencies();

	--Print output to let the player know the command succeeded, if there is no UI
	MCom.printStatus(BAROPTIONS_CHAT_SIDEBARSETUP);
end;

--[[ Moves Party Frame out of left sidebar's way ]]--
BarOptions.OffsetParty = function (doOffset)
	local newXPos = 10;
	--if we are supposed to offset party frame for the left bar, then calculate the new position
	if(doOffset) then
		local width = MultiBarLeft:GetWidth();
		local scale = MultiBarLeft:GetEffectiveScale();
		if( ( width ) and ( scale ) ) then
			newXPos = newXPos + (width * scale);
		end
	end

	--Move the party member frame to its new home
	PartyMemberFrame1:ClearAllPoints();
	PartyMemberFrame1:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", newXPos, -128);
	if(doOffset) then
		PartyMemberFrame1:SetUserPlaced(true);
	else
		PartyMemberFrame1:SetUserPlaced(false);
	end
end;

--[[ Moves Main Bar ]]--
BarOptions.MoveMainBar = function ()
	--Default to not moving it
	local mainPoint = nil;
	--If any of the movers are enabled, then set mainPoint for it
	if( BarOptions_Config.MainBarCenter == 1 ) then
		mainPoint = "BOTTOM";
	elseif( BarOptions_Config.MainBarLeft == 1 ) then
		mainPoint = "BOTTOMLEFT";
	elseif( BarOptions_Config.MainBarRight == 1 ) then
		mainPoint = "BOTTOMRIGHT";
	end

	--Adjust for TitanBar
	local yOff = 0;
	if(TitanPanelGetVar and TITAN_PANEL_PLACE_BOTTOM and ( ( TitanPanelGetVar("Position") == TITAN_PANEL_PLACE_BOTTOM ) or TitanPanelGetVar("BothBars") ) and TitanMovable_GetPanelYOffset ) then
		local titanOffset = TitanMovable_GetPanelYOffset(TITAN_PANEL_PLACE_BOTTOM, TitanPanelGetVar("BothBars"));
		if(titanOffset) then
			yOff = yOff + titanOffset;
		end
	end

	--Move the main bar to the new position, if needed
	if( mainPoint and ( not MainMenuBar:IsUserPlaced() ) ) then
		MainMenuBar:ClearAllPoints();
		MainMenuBar:SetPoint(mainPoint, "UIParent", mainPoint, 0, yOff);
	end
end;

--[[ Shows or hides a frame ]]--
BarOptions.ShowFrame = function (whichFrame, hideShow)
	--Show/hide the frame
	if(hideShow == 1) then
		if(not getglobal(whichFrame):IsVisible()) then
			getglobal(whichFrame):Show();
		end
	else
		if(getglobal(whichFrame):IsVisible()) then
			getglobal(whichFrame):Hide();
		end
	end
end;

--[[ Updates all hotkey text ]]--
BarOptions.UpdateHotkeys = function ()
	for num, name in ipairs(BarOptions.BARLIST) do
		for curBut=1, 12 do
			BarOptions.ActionButton_UpdateHotkeys(nil, getglobal(name.."Button"..curBut));
		end
	end
end;

--[[ Updates showing of multibar hotkey text ]]--
BarOptions.ShowMultiHotkeys = function ()
	for index = 3, getn(BarOptions.BARLIST) do
		for curBut=1, 12 do
			local name = BarOptions.BARLIST[index];
			BarOptions.ShowFrame(name.."Button"..curBut.."HotKey", BarOptions_Config.MultiKeys);
		end
	end
end;

--[[ Shows or hides a button based on its ID and parent ]]--
BarOptions.ShowButton = function (button)
	if(not button) then
		button = this;
	end
	local shouldShow = 1;

	local buttonID = button:GetID();
	if(not buttonID or buttonID < 1) then
		buttonID = (SecureButton_GetModifiedAttribute(button, "action", SecureStateChild_GetEffectiveButton(button)) or 1);
		if(buttonID > 12) then
			buttonID = math.fmod(buttonID, 12);
			if(buttonID == 0) then
				buttonID = 12;
			end
		end
	end

	if(string.find(button:GetName(), "MultiBarBottomLeft")) then
		if(buttonID > BarOptions_Config.BLCount) then
			shouldShow = 0;
		end
	elseif(string.find(button:GetName(), "MultiBarBottomRight")) then
		if(buttonID > BarOptions_Config.BRCount) then
			shouldShow = 0;
		end
	elseif(string.find(button:GetName(), "MultiBarRight")) then
		if(buttonID > BarOptions_Config.RCount) then
			shouldShow = 0;
		end
	elseif(string.find(button:GetName(), "MultiBarLeft")) then
		if(buttonID > BarOptions_Config.LCount) then
			shouldShow = 0;
		end
	end

	if( button:GetAttribute("statehidden") ) then
		shouldShow = 0;
	end

	if( button.showgrid == 0 and not HasAction(ActionButton_GetPagedID(button)) and BarOptions_Config.HideEmpty == 1) then
		shouldShow = 0;
	end

	if( BarOptions_Config.HideEmpty == 0 ) then
		shouldShow = 1;
	end

	if(shouldShow == 1) then
		if(not button:IsVisible()) then
			if( (string.find(button:GetName(), "ArtButton")) or HasAction(ActionButton_GetPagedID(button)) or (button.showgrid > 0) or (BarOptions_Config.HideEmpty == 0) ) then
				button:Show();
			end
		end
	elseif(button:IsVisible()) then
		button:Hide();
	end
end;

--[[ Updates showing/hiding of buttons on specified multibar ]]--
BarOptions.UpdateHiddenStates = function(bar)
	if( (not bar) or (bar < 1) or (bar > 4) ) then return; end

	local counts = { "BLCount", "BRCount", "RCount", "LCount" };

	for curBut=1, 12 do
		local name = BarOptions.BARLIST[bar+2];
		local button = getglobal(name.."Button"..curBut);
		if(curBut > BarOptions_Config[counts[bar]] and (button:GetAttribute("hiddenstates") ~= "*")) then
			button:SetAttribute("hiddenstates", "*");
			button:SetAttribute("statehidden", true);
		else
			button:SetAttribute("hiddenstates", nil);
			button:SetAttribute("statehidden", nil);
		end
		BarOptions.ShowButton(button);
		button = getglobal(name.."ArtButton"..curBut);
		BarOptions.ShowButton(button);
	end
end

--[[ Updates showing of all buttons ]]--
BarOptions.UpdateButtons = function ()
	for index = 1, #BarOptions.BARLIST do
		if(index > 2) then
			BarOptions.UpdateHiddenStates(index-2);
		else
			for curBut=1, 12 do
				local name = BarOptions.BARLIST[index];
				BarOptions.ShowButton(getglobal(name.."Button"..curBut));
			end
		end
	end
end;

--[[ Updates showing of all artwork ]]--
BarOptions.UpdateArt = function ()
	--Show/hide MainBar dragons
	BarOptions.ShowFrame("MainMenuBarLeftEndCap", BarOptions_Config.MainArt);
	BarOptions.ShowFrame("MainMenuBarRightEndCap", BarOptions_Config.MainArt);

	--Show/hide Multi Bar Artwork
	BarOptions.ShowFrame("MultiBarBottomLeftArt", BarOptions_Config.BLArt);
	BarOptions.ShowFrame("MultiBarBottomRightArt", BarOptions_Config.BRArt);
	BarOptions.ShowFrame("MultiBarRightArt", BarOptions_Config.RArt);
	BarOptions.ShowFrame("MultiBarLeftArt", BarOptions_Config.LArt);
end;

--[[ Saves the current configuration on a per realm/per character basis ]]--
BarOptions.SaveConfig = function ()
	--Use MCom's save function to save the config
	MCom.saveConfig( {
		configVar = "BarOptions_Config";
	});
end

--[[ Loads the current configuration from a per realm/per character variable set ]]--
BarOptions.LoadConfig = function ()
	--Use MCom's load function to load the config
	MCom.loadConfig( {
		configVar = "BarOptions_Config";
		nonUIList = {};
	});
end

--[[ Keybinding handler for instant action bars ]]--
BarOptions.BindingHandler = function(page)
		if( GetActionBarPage() ~= page ) then
			if(Instant_ActionBar_Return == -1 ) then
				Instant_ActionBar_Return = GetActionBarPage();
			end
			ChangeActionBarPage(page);
		end
end

--[[ Creates SecureStateHeaders to drive our action bars ]]--
BarOptions.SetupSecureStateDrivers = function()
	local _,playerClass = UnitClass("player");
	if(not playerClass) then
		playerClass = "noclass";
	end

	if(not BarOptions.SecureStateDrivers) then
		BarOptions.SecureStateDrivers = {};
		local parent = UIParent;
		for i = 1, #BarOptions.BARLIST do
			if(i == 1) then
				parent = MainMenuBarArtFrame;
			elseif(i == 2) then
				parent = BonusActionBarFrame;
			elseif(i > 2) then
				parent = getglobal(BarOptions.BARLIST[i]);
			end
			BarOptions.SecureStateDrivers[BarOptions.BARLIST[i]] = CreateFrame("Frame", nil, parent, "SecureStateDriverTemplate");
			(BarOptions.SecureStateDrivers[BarOptions.BARLIST[i]]):SetAttribute("state", 1);
		end
	end

	-- Let's fix up the current state on the main action bar and the bottom left multi action bar
	--
	--	Some background info for those that wish to know
	--		Default Stance/Shapeshift info:
	--			Warrior:	0: not possible*	1: battle			2: defensive	3: berserker
	--			Druid:		0: caster					1: bear				2: aquatic		3: cat				4: travel		5: moonkin/tree of life		6: flight(5 without moonkin/tree of life)
	--			Rogue:		0: not stealthed	1: stealthed
	--			Priest:		0: normal					1: shadowform
	--
	--			(*: on some rare conditions a warrior may be forced out of any stance, in which case they will then be in stance 0)
	--
	--	Due to Blizzard only providing 120 action button ID's, it is only possible to use at most 10 action bar pages of 12 buttons each
	--	ID page ranges 1 through 6 are used by the blizzard 6 action bars and the 4 multi bars.
	--	ID Pages 7,8,9 are used by some classes.  Page 10 isn't used by anything in default UI.

	--
	local _page, _stance, _stealthed = GetActionBarPage(), GetShapeshiftForm(true), (IsStealthed() or 0);

	-- Start by getting the current action bar page(1 on fresh login, 1-6 if reloading UI)
	local currentState = _page;
	-- as well as the current stance number
	local stanceNumber = _stance;

	-- Remap druid forms to compress them based on options the users config
	if(playerClass == "DRUID") then
		-- convert aquatic/travel/flight form back to caster
		--(temp: also moonkin/tree until option for their stances is added)
		if((stanceNumber == 2) or (stanceNumber == 4) or (stanceNumber == 5) or (stanceNumber == 6)) then
			stanceNumber = 0;
		end
		-- if we're in cat, and we want a stealth bar, and we're stealthed, set the stance to 2
		if((stanceNumber == 3) and (BarOptions_Config.DruidStealth == 1) and IsStealthed()) then
			stanceNumber = 2;
		end
	end

	currentState = currentState+(stanceNumber*6);

	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-actionbar-1", "1-6:1;7-12:7;13-18:13;19-24:19");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-actionbar-2", "1-6:2;7-12:8;13-18:14;19-24:20");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-actionbar-3", "1-6:3;7-12:9;13-18:15;19-24:21");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-actionbar-4", "1-6:4;7-12:10;13-18:16;19-24:22");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-actionbar-5", "1-6:5;7-12:11;13-18:17;19-24:23");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-actionbar-6", "1-6:6;7-12:12;13-18:18;19-24:24");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-stance-0", "7:1;13:1;19:1;8:2;14:2;20:2;9:3;15:3;21:3;10:4;16:4;22:4;11:5;17:5;23:5;12:6;18:6;24:6;*:1");
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-stance-1", "1:7;13:7;19:7;2:8;14:8;20:8;3:9;15:9;21:9;4:10;16:10;22:10;5:11;17:11;23:11;6:12;18:12;24:12;*:=");
	if(playerClass == "DRUID") then
		if(BarOptions_Config.DruidStealth == 1) then
			BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-stealth-1", "19:13;20:14;21:15;22:16;23:17;24:18;*:=");
			BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-stealth-0", "13:19;14:20;15:21;16:22;17:23;18:24;*:=");
		end
	else
		BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-stance-2", "1:13;7:13;19:13;2:14;8:14;20:14;3:15;9:15;21:15;4:16;10:16;22:16;5:17;11:17;23:17;6:18;12:18;24:18;*:=");
	end
	BarOptions.SecureStateDrivers.Action:SetAttribute("statemap-stance-3", "1:19;7:19;13:19;2:20;8:20;14:20;3:21;9:21;15:21;4:22;10:22;16:22;5:23;11:23;17:23;6:24;12:24;18:24;*:=");

	BarOptions.SecureStateDrivers.Action:SetAttribute("state", currentState);
	BarOptions.SecureStateDrivers.Action:SetAttribute("state-actionbar", _page);
	BarOptions.SecureStateDrivers.Action:SetAttribute("state-stance", _stance);
	BarOptions.SecureStateDrivers.Action:SetAttribute("state-stealth", _stealthed);

	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-actionbar-1", "1-6:1;7-12:7;13-18:13;19-24:19");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-actionbar-2", "1-6:2;7-12:8;13-18:14;19-24:20");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-actionbar-3", "1-6:3;7-12:9;13-18:15;19-24:21");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-actionbar-4", "1-6:4;7-12:10;13-18:16;19-24:22");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-actionbar-5", "1-6:5;7-12:11;13-18:17;19-24:23");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-actionbar-6", "1-6:6;7-12:12;13-18:18;19-24:24");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-stance-0", "7:1;13:1;19:1;8:2;14:2;20:2;9:3;15:3;21:3;10:4;16:4;22:4;11:5;17:5;23:5;12:6;18:6;24:6;*:1");
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-stance-1", "1:7;13:7;19:7;2:8;14:8;20:8;3:9;15:9;21:9;4:10;16:10;22:10;5:11;17:11;23:11;6:12;18:12;24:12;*:=");
	if((playerClass == "DRUID") and (BarOptions_Config.DruidStealth == 1)) then
		BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-stealth-1", "19:13;20:14;21:15;22:16;23:17;24:18;*:=");
		BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-stealth-0", "13:19;14:20;15:21;16:22;17:23;18:24;*:=");
	else
		BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-stance-2", "1:13;7:13;19:13;2:14;8:14;20:14;3:15;9:15;21:15;4:16;10:16;22:16;5:17;11:17;23:17;6:18;12:18;24:18;*:=");
	end
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("statemap-stance-3", "1:19;7:19;13:19;2:20;8:20;14:20;3:21;9:21;15:21;4:22;10:22;16:22;5:23;11:23;17:23;6:24;12:24;18:24;*:=");

	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("state", currentState);
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("state-actionbar", _page);
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("state-stance", _stance);
	BarOptions.SecureStateDrivers.MultiBarBottomLeft:SetAttribute("state-stealth", _stealthed);
end

--[[ Local tables used in the action button ID functions following ]]--
local action_stances = {};
action_stances["DRUID"] = {9, 8, 7};
action_stances["WARRIOR"] = {7, 8, 9};
action_stances["ROGUE"] = {7};
action_stances["PRIEST"] = {7};

local multibar_stances = {};
multibar_stances["DRUID"] = {BOTTOMLEFT_ACTIONBAR_PAGE, 10, 10, 10};
multibar_stances["WARRIOR"] = {BOTTOMLEFT_ACTIONBAR_PAGE, BOTTOMLEFT_ACTIONBAR_PAGE, 1, 10};
multibar_stances["ROGUE"] = {BOTTOMLEFT_ACTIONBAR_PAGE, 8, 8, 8};
multibar_stances["PRIEST"] = {BOTTOMLEFT_ACTIONBAR_PAGE, 8, 8, 8};

local alt_multibar_stances = {};
alt_multibar_stances["DRUID"] = {ALT_BOTTOMLEFT_ACTIONBAR_PAGE, 10, 10, 10};
alt_multibar_stances["WARRIOR"] = {ALT_BOTTOMLEFT_ACTIONBAR_PAGE, ALT_BOTTOMLEFT_ACTIONBAR_PAGE, 1, 10};
alt_multibar_stances["ROGUE"] = {ALT_BOTTOMLEFT_ACTIONBAR_PAGE, 8, 8, 8};
alt_multibar_stances["PRIEST"] = {ALT_BOTTOMLEFT_ACTIONBAR_PAGE, 8, 8, 8};

--[[ Sets up one special case or all four of the states for the bottom left multi bar ]]--
BarOptions.SetupMultiBarButtonIDs = function(stancePage)
	local _,playerClass = UnitClass("player");
	if(not playerClass) then
		playerClass = "noclass";
	end

	local extraButtons, groupedButtons = 0, NUM_ACTIONBAR_BUTTONS;
	if((BarOptions_Config.AlternateIDs == 1) and (BarOptions_Config.TurnPages == 1) and (BarOptions_Config.GroupPages == 1)) then
		groupedButtons = groupedButtons * 2;
		extraButtons = NUM_ACTIONBAR_BUTTONS;
	end

	local stance_pages = multibar_stances;
	if(BarOptions_Config.AlternateIDs == 1) then
		stance_pages = alt_multibar_stances
	end

	custom_stances = {};
	if(BarOptions_Config.CustomStances == 1) then
		custom_stances[0] = MCom.getStringVar("BarOptions_Config.Stance0");
		custom_stances[1] = MCom.getStringVar("BarOptions_Config.Stance1");
		custom_stances[2] = MCom.getStringVar("BarOptions_Config.Stance2");
		custom_stances[3] = MCom.getStringVar("BarOptions_Config.Stance3");
	end

	local lowStance, highStance = 0, 3;
	if(stancePage and stancePage >=0 and stancePage <=3) then
		lowStance, highStance = stancePage, stancePage;
	end

	local button, page = nil, BOTTOMLEFT_ACTIONBAR_PAGE;
	for i = 1, 12 do
		button = getglobal("MultiBarBottomLeftButton"..i);
		for y = lowStance, highStance do
			for z = 1, 6 do
				if(BarOptions_Config.AlternateIDs == 1) then
					page = ALT_BOTTOMLEFT_ACTIONBAR_PAGE;
					if(BarOptions_Config.TurnPages == 1) then
						page = (z+1);
						if(page > 6) then
							page = 1;
						end
						if(BarOptions_Config.GroupPages == 1) then
							page = z;
							if(page > 3) then
								page = page - 3;
							end
						end
					end
				elseif((BarOptions_Config.StanceBar == 1) and (BarOptions_Config.TurnPages == 0) and (BarOptions_Config.GroupPages == 0)) then
					if(custom_stances[y]) then
						page = custom_stances[y];
					elseif(stance_pages[playerClass] and stance_pages[playerClass][y+1]) then
						page = stance_pages[playerClass][y+1];
					end
				else
					page = BOTTOMLEFT_ACTIONBAR_PAGE;
				end
				button:SetAttribute("*action-S"..((y*6)+z).."S", (i+extraButtons+((page-1)*groupedButtons)));
				button:SetID(0);
			end
		end
	end
end

--[[ Sets up the main action bar IDs, 24 states ]]--
BarOptions.SetupActionBarButtonIDs = function()
	local _,playerClass = UnitClass("player");
	if(not playerClass) then
		playerClass = "noclass";
	end

	local button, page;

	local groupedButtons = NUM_ACTIONBAR_BUTTONS;
	for i = 1, 12 do
		button = getglobal("ActionButton"..i);
		for y = 0, 3 do
			for z = 1, 6 do
				if((z == 1) and action_stances[playerClass] and action_stances[playerClass][y]) then
					page = action_stances[playerClass][y];
				else
					page = z;
				end
				if(z == 1) then
					groupedButtons = NUM_ACTIONBAR_BUTTONS;
				else
					if((BarOptions_Config.AlternateIDs == 1) and (BarOptions_Config.TurnPages == 1) and (BarOptions_Config.GroupPages == 1)) then
						groupedButtons = NUM_ACTIONBAR_BUTTONS * 2;
					end
				end
				button:SetAttribute("*action-S"..((y*6)+z).."S", (i+((page-1)*groupedButtons)));
				button:SetID(0);
			end
		end
	end
end

--[[ Sets up all the action buttons with the correct IDs for each state ]]--
BarOptions.SetupActionButtonIDs = function()
	local button;

	for b = 1, 6 do
		if(BarOptions.BARLIST[b] == "Action") then
			BarOptions.SetupActionBarButtonIDs();
		elseif(BarOptions.BARLIST[b] == "MultiBarBottomLeft") then
			BarOptions.SetupMultiBarButtonIDs();
		else
			for i = 1, 12 do
				button = getglobal(BarOptions.BARLIST[b].."Button"..i);
				if(BarOptions.BARLIST[b] == "MultiBarBottomRight") then
					if(BarOptions_Config.AlternateIDs == 0) then
						--Normal IDs
						button:SetAttribute("*action-S1S", (i+((BOTTOMRIGHT_ACTIONBAR_PAGE-1)*NUM_ACTIONBAR_BUTTONS)));
					else
						--Alternate IDs
						button:SetAttribute("*action-S1S", (i+((ALT_BOTTOMRIGHT_ACTIONBAR_PAGE-1)*NUM_ACTIONBAR_BUTTONS)));
					end
				elseif(BarOptions.BARLIST[b] == "MultiBarLeft") then
					if(BarOptions_Config.AlternateIDs == 0) then
						--Normal IDs
						button:SetAttribute("*action-S1S", (i+((LEFT_ACTIONBAR_PAGE-1)*NUM_ACTIONBAR_BUTTONS)));
					else
						--Alternate IDs
						if(i <= 6) then
							--Map the last 6 of the last 12 IDs to the first 6 buttons
							button:SetAttribute("*action-S1S", (i+114));
						else
							--Map the last 6 of the specified ID range to the last 6 buttons
							button:SetAttribute("*action-S1S", (i+((ALT_SIDEBAR_PAGE-1)*NUM_ACTIONBAR_BUTTONS)));
						end
					end
				elseif(BarOptions.BARLIST[b] == "MultiBarRight") then
					if(BarOptions_Config.AlternateIDs == 0) then
						--Normal IDs
						button:SetAttribute("*action-S1S", (i+((RIGHT_ACTIONBAR_PAGE-1)*NUM_ACTIONBAR_BUTTONS)));
					else
						--Alternate IDs
						if(i <= 6) then
							--Map the first 6 of the last 12 IDs to the first 6 buttons
							button:SetAttribute("*action-S1S", (i+108));
						else
							--Map the first 6 of the specified ID range to the last 6 buttons
							button:SetAttribute("*action-S1S", (i+((ALT_SIDEBAR_PAGE-1)*NUM_ACTIONBAR_BUTTONS)-(NUM_ACTIONBAR_BUTTONS/2)));
						end
					end
				else
					button:SetAttribute("*action-S1S", i);
				end
				button:SetID(0);
			end
		end
	end
end

--[[ Converts default Blizzard action buttons to use new 2.0 secure attributes,
			also hides the BonusActionBar since that is no longer needed ]]--
BarOptions.SetupActionButtons = function()
	local button;

	local virtual_button_text = "";
	for z = 1, 24 do
		virtual_button_text = virtual_button_text..z..":S"..z.."S";
		if(z < 24) then
			virtual_button_text = virtual_button_text..";";
		end
	end

	for b = 1, 6 do
		driver = BarOptions.SecureStateDrivers[BarOptions.BARLIST[b]];
		for i = 1, 12 do
			button = getglobal(BarOptions.BARLIST[b].."Button"..i);
			button:SetScript("OnAttributeChanged", ActionButton_Update);
			driver:SetAttribute("addchild", button);
			if(ShiftFixClick_ActionButton) then ShiftFixClick_ActionButton(button); end
			if(BarOptions.BARLIST[b] == "BonusAction") then
				--Hide the bonus action bar buttons since it's function has been emulated in the main action bar now(1 state, hidden)
				button:SetAttribute("*statebutton1", "1:S1S");
				button:SetAttribute("*type-S1S", "action");
				button:SetAttribute("hiddenstates", "*");
				button:SetAttribute("statehidden", true);
			elseif((BarOptions.BARLIST[b] == "Action") or (BarOptions.BARLIST[b] == "MultiBarBottomLeft")) then
				--Set the main action bar to act like the old bonus action bar in addition to switching action pages(24 states, 4 stances at 6 pages per stance)
				--Also setup the the bottom left action bar to allow for stance bars if selected.
				button:SetAttribute("*statebutton1", virtual_button_text);
				for y = 0, 3 do
					for z = 1, 6 do
						button:SetAttribute("*type-S"..((y*6)+z).."S", "action");
					end
				end
			else
				--Everything else just setup one state
				button:SetAttribute("*statebutton1", "1:S1S");
				button:SetAttribute("*type-S1S", "action");
			end
			button:UnregisterEvent("UPDATE_BONUS_ACTIONBAR");
			button:SetID(0);
		end
	end
	--Since the BonusActionBar is no longer needed, replace most of it's functions with stubs and unregister/hide it
	BonusActionBar_OnEvent = function() end
	BonusActionBar_OnUpdate = function(elapsed) end
	--BonusActionButton_OnEvent = function() end
	--BonusActionButtonUp = function(id) end
	--BonusActionButtonDown = function(id) end
	BonusActionBar_SetButtonTransitionState = function(state) end
	ShowBonusActionBar = function() end
	HideBonusActionBar = function() end
	BonusActionBarFrame:Hide();
	BonusActionBarFrame:UnregisterEvent("UPDATE_BONUS_ACTIONBAR");
	BonusActionBarFrame:UnregisterEvent("ACTIONBAR_SHOWGRID");
	BonusActionBarFrame:UnregisterEvent("ACTIONBAR_HIDEGRID");
end

--------------------------------------------------
--
-- Event Handlers
--
--------------------------------------------------
BarOptions.OnVarsLoaded = function ()
	if(not BarOptions.ConfigLoaded) then
		BarOptions.ConfigLoaded = true;
		--Load the configuration
		BarOptions.LoadConfig();
		--Store the configuration for this character
		BarOptions.SaveConfig();

		--Setup the action buttons with the correct IDs
		BarOptions.SetupActionButtonIDs();

		--Update the main bar position
		BarOptions.MoveMainBar();
		--Update the artwork
		BarOptions.UpdateArt();
		--Update which buttons should be showing
		BarOptions.UpdateButtons();
		--Update showing of the Hotkeys
		BarOptions.UpdateHotkeys();
		BarOptions.ShowMultiHotkeys();
	end;
end

BarOptions.OnEvent = function(self, event)
	if(event == "PLAYER_REGEN_ENABLED") then
		--after player leaves combat check to see if a button should be visible or not(bug fix: hide empty buttons after changing action pages in combat)
		BarOptions.UpdateButtons();
	end
end


BarOptions.OnLoad = function ()
	if( BarOptions.Initialized ~= 1) then
		--Setup Secure State Headers to control the default action bars
		BarOptions.SetupSecureStateDrivers();
		--Setup the default action buttons to use the new secure attribute methods
		BarOptions.SetupActionButtons();

		--Hook the functions that need hooked
		if(BarOptions.Hook) then
			BarOptions.Hook();
		end

		--Set these frames mobile so they can retain positions if needed
		MultiBarRight:SetMovable(true);
		MultiBarLeft:SetMovable(true);
		PartyMemberFrame1:SetMovable(true);

		--Register the configuration options
		BarOptions.Register();

		--Register to be informed when the vars needed for config are loaded
		MCom.registerVarsLoaded(BarOptions.OnVarsLoaded);

		BarOptions.Initialized = 1;

		BarOptionsFrame:RegisterEvent("PLAYER_REGEN_ENABLED");
		BarOptionsFrame:SetScript("OnEvent", BarOptions.OnEvent);
	end
end;
